@@ -7,7 +7,7 @@
 *
 *	This file part of:	SWarp
 *
-*	Copyright:		(C) 2000-2014 Emmanuel Bertin -- IAP/CNRS/UPMC
+*	Copyright:		(C) 2000-2020 IAP/CNRS/SorbonneU
 *
 *	License:		GNU General Public License
 *
@@ -22,7 +22,7 @@
 *	You should have received a copy of the GNU General Public License
 *	along with SWarp. If not, see <http://www.gnu.org/licenses/>.
 *
-*	Last modified:		10/03/2014
+*	Last modified:		03/01/2020
 *
 *%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
@@ -39,6 +39,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "define.h"
 #include "types.h"
@@ -120,6 +121,158 @@ static void	max_clique_recur(unsigned int *array, int nnode, int *old,
 		*pthread_move_lines(void *arg);
 #endif
 
+#ifdef HAVE_CFITSIO
+
+ /**
+  * CFITSIO
+  *
+  * Function to create a cfitsio tile-compressed image.
+  * Uses the existing filename with an 'fz' extension.
+  * Currently, cfitsio compression defaults are used:
+  * - each tile is one row of the image
+  * - Rice compression is used
+  */
+ int setupTileCompressedFile(fieldstruct *outfield, int disallow_negative_pixels) {
+
+	if (!prefs.tile_compress_flag) {
+
+		outfield->cat->tab->infptr = NULL;
+ 		//printf("DEBUG CFITSIO NOT creating tile-compressed output image file\n");
+		return 0;
+	}
+
+ 	// create a file-name: use existing name with '.fz' suffix
+ 	char compressedFilename[strlen(outfield->cat->filename) + 20];
+ 	sprintf(compressedFilename, "!%s.fz[compress]", outfield->cat->filename);
+
+ 	// now create file
+ 	int status = 0; fits_create_file(&outfield->cat->tab->infptr, compressedFilename, &status);
+ 	if (status != 0) {
+
+ 		printf("CFITSIO ERROR creating output image file\n");
+ 		fits_report_error(stderr, status);
+ 		return 0;
+ 	}
+
+ 	// now create image extension
+ 	long naxis[2] = {outfield->cat->tab->naxisn[0], outfield->cat->tab->naxisn[1]};
+ 
+	if (disallow_negative_pixels) {
+          // set quantisation method. We can use this to prevent <0 pixel values
+	  status = 0;
+#if CFITSIO_MAJOR >= 3
+  #if CFITSIO_MINOR >= 35
+	  fits_set_quantize_method(outfield->cat->tab->infptr, SUBTRACTIVE_DITHER_2, &status);
+  #else
+	  fits_set_quantize_dither(outfield->cat->tab->infptr, SUBTRACTIVE_DITHER_1, &status);
+  #endif
+#endif
+ 	  if (status != 0) {
+
+ 	  	printf("CFITSIO ERROR error setting CFitsIO quantize method\n");
+ 	  	fits_report_error(stderr, status);
+ 	  	return 0;
+ 	  }
+        }
+
+	// actually create the image
+	status = 0; fits_create_img(outfield->cat->tab->infptr, FLOAT_IMG, 2, naxis, &status);
+ 	if (status != 0) {
+
+ 		printf("CFITSIO ERROR creating output image extension\n");
+ 		fits_report_error(stderr, status);
+ 		return 0;
+ 	}
+
+ 	// point to the first element in the image, ready for writing
+ 	outfield->tab->currentElement = 1;
+
+ 	//printf("DEBUG CFITSIO successfully created tile-compressed output image file, %s\n", compressedFilename);
+
+ 	// now, change original filename to temp one (.tmp suffix)
+ 	sprintf(outfield->cat->filename, "%s.tmp", outfield->cat->filename);
+
+ 	return 1;
+ }
+
+ /**
+   * CFITSIO
+   *
+   * Function to copy all non-structural keywords from one FITS file to tile-compressed one
+   *
+   */
+ int copyHeaderToTileCompressedFile(fieldstruct *outfield) {
+
+	 if (outfield->tab->infptr == NULL) return 0;
+
+	 // use cfitsio to open input file from which to copy header
+	 fitsfile *infptr;
+	 int status = 0; fits_open_file(&infptr, outfield->cat->filename, READONLY, &status);
+	 if (status != 0) {
+
+		 printf("ERROR cfitsio could not open FITS file for header copying: %s\n", outfield->cat->filename);
+		 fits_report_error(stderr, status);
+		 return 0;
+	 }
+
+	 // copy headers
+	 int nkeys;
+	 fits_get_hdrspace(infptr, &nkeys, NULL, &status);
+
+	 // loop through and copy non-structural keywords
+	 int i;
+	 char card[81];
+	 for (i=1; i<=nkeys; i++) {
+
+		 fits_read_record(infptr, i, card, &status);
+
+		 if (fits_get_keyclass(card) > TYP_CMPRS_KEY) {
+
+			 fits_write_record(outfield->tab->infptr, card, &status);
+			 //printf("DEBUG    WRITING %s\n", card);
+		 }
+		 //else
+	     //printf("DEBUG NOT WRITING %s\n", card);
+	 }
+
+	 return 1;
+ }
+
+ /**
+   * CFITSIO
+   *
+   * Function to close a cfitsio-created tile-compressed image.
+   * Also deletes temp non-tile-compressed file used to copy header keywords.
+   *
+   */
+  int closeTileCompressedFile(fieldstruct *outfield) {
+
+ 	 if (outfield->tab->infptr == NULL) return 0;
+
+ 	 // first copy header from non-tile compressed file
+ 	 copyHeaderToTileCompressedFile(outfield);
+
+ 	 int status = 0; fits_close_file(outfield->tab->infptr, &status);
+ 	 if (status != 0) {
+
+ 		 printf("CFITSIO ERROR closing tile-compressed image file\n");
+ 		 fits_report_error(stderr, status);
+ 		 return 0;
+ 	 }
+
+ 	 //printf("CFITSIO successfully closed tile-compressed output image file\n");
+
+ 	 // now delete temp file
+ 	 if (unlink (outfield->cat->filename) != 0) {
+
+ 		 printf("CFITSIO ERROR could not delete %s\n", outfield->cat->filename);
+ 	 }
+
+ 	 return 1;
+  }
+
+#endif // HAVE_CFITSIO
+
 /******* coadd_fields *********************************************************
 PROTO	int coadd_fields(fieldstruct **infield, fieldstruct **inwfield,
 			int ninput,
@@ -164,6 +317,12 @@ int coadd_fields(fieldstruct **infield, fieldstruct **inwfield, int ninput,
 			nbuflines,nbuflines2,nbuflinesmax, size, omax,omax2,
 			offbeg, offend, fieldno, nopenfiles, closeflag;
 
+#ifdef HAVE_CFITSIO
+  // CFITSIO set up tile compressed output images (if specified by user)
+  setupTileCompressedFile(outfield, 0);
+  setupTileCompressedFile(outwfield, 1);
+#endif // HAVE_CFITSIO
+
   coadd_type = coaddtype;
   coadd_wthresh = wthresh;
   naxis = outfield->tab->naxis;
@@ -743,6 +902,14 @@ int coadd_fields(fieldstruct **infield, fieldstruct **inwfield, int ninput,
     free(multinbuf);
     }
 
+#ifdef HAVE_CFITSIO
+  // CFITSIO close tile compressed files
+  if (prefs.tile_compress_flag) {
+
+	  closeTileCompressedFile(outfield);
+	  closeTileCompressedFile(outwfield);
+  }
+#endif // HAVE_CFITSIO
   return RETURN_OK;
   }
 
@@ -1091,7 +1258,7 @@ INPUT	Current line number within the buffer,
 OUTPUT	RETURN_OK if no error, or RETURN_ERROR in case of non-fatal error(s).
 NOTES   Requires many global variables (for multithreading).
 AUTHOR  E. Bertin (IAP), D. Gruen (USM)
-VERSION	10/03/2014
+VERSION	23/10/2015
  ***/
 int	coadd_line(int l, int b, int *bufmin)
 
@@ -1127,7 +1294,7 @@ int	coadd_line(int l, int b, int *bufmin)
           {
           fval2 = *(inpixt++);
           wval2 = *(inwpixt++);
-          if (wval2<coadd_wthresh)
+          if (wval2<coadd_wthresh && fval2>-BIG/2)
             {
             ninput2++;
             wval += (wval2=1.0/wval2);
@@ -1174,7 +1341,7 @@ int	coadd_line(int l, int b, int *bufmin)
           wval2 = *(inwpixt++);
           origin2 = *(inorigint++);
 
-          if (wval2<coadd_wthresh)
+          if (wval2<coadd_wthresh && val2>-BIG/2)
             {
             ninput2++;
             *(pixt++) = val2;
@@ -1262,7 +1429,7 @@ int	coadd_line(int l, int b, int *bufmin)
 				(fabsf(fabsf(*(pixstack+i) - mu) -
 				prefs.clip_ampfrac*amu)) / sigmaeff);
               }
-            if (wval > 0.0)
+            if (wval > 0.0 && val>-BIG/2)
               {
               *(outwpix++) = (wval = 1.0/wval);
               *(outpix++) = val*wval; 
@@ -1294,7 +1461,7 @@ int	coadd_line(int l, int b, int *bufmin)
           {
           fval2 = *(inpixt++);
           wval2 = *(inwpixt++);        
-          if (wval2 < coadd_wthresh)
+          if (wval2 < coadd_wthresh && fval2>-BIG/2)
             {
             *(pixt++) = fval2;
             wval += 1.0/sqrt(wval2);
@@ -1334,7 +1501,7 @@ int	coadd_line(int l, int b, int *bufmin)
           {
           fval2 = *(inpixt++);
           wval2 = *(inwpixt++);
-          if (wval2<coadd_wthresh)
+          if (wval2<coadd_wthresh && fval2>-BIG/2)
             {
             ninput2++;
             val += fval2;
@@ -1368,7 +1535,7 @@ int	coadd_line(int l, int b, int *bufmin)
           {
           val2 = *(inpixt++);
           wval2 = *(inwpixt++);
-          if (wval2<coadd_wthresh)
+          if (wval2<coadd_wthresh && val2>-BIG/2)
             {
             ninput2++;
             if (val2<val)
@@ -1401,7 +1568,7 @@ int	coadd_line(int l, int b, int *bufmin)
           {
           val2 = *(inpixt++);
           wval2 = *(inwpixt++);
-          if (wval2<coadd_wthresh)
+          if (wval2<coadd_wthresh && val2>-BIG/2)
             {
             ninput2++;
             if (val2>val)
@@ -1436,7 +1603,7 @@ int	coadd_line(int l, int b, int *bufmin)
           {
           val2 = *(inpixt++);
           wval2 = *(inwpixt++);
-          if (wval2<coadd_wthresh)
+          if (wval2<coadd_wthresh && val2>-BIG/2)
             {
             ninput2++;
             wval0 = 1.0/wval2;
@@ -1472,7 +1639,7 @@ int	coadd_line(int l, int b, int *bufmin)
           {
           val2 = *(inpixt++);
           wval2 = *(inwpixt++);
-          if (wval2<coadd_wthresh)
+          if (wval2<coadd_wthresh && val2>-BIG/2)
             {
             ninput2++;
             val0 = val2;
@@ -1512,7 +1679,7 @@ int	coadd_line(int l, int b, int *bufmin)
           {
           val2 = *(inpixt++);
           wval2 = *(inwpixt++);
-          if (wval2<coadd_wthresh)
+          if (wval2<coadd_wthresh && val2>-BIG/2)
             {
             ninput2++;
             val0 = val2;
@@ -1550,7 +1717,7 @@ int	coadd_line(int l, int b, int *bufmin)
           {
           val2 = *(inpixt++);
           wval2 = *(inwpixt++);
-          if (wval2<coadd_wthresh)
+          if (wval2<coadd_wthresh && val2>-BIG/2)
             {
             ninput2++;
             val += val2;
@@ -1796,7 +1963,7 @@ int	coadd_iload(fieldstruct *field, fieldstruct *wfield,
 			int nbuflines, int outwidth, int multinmax)
   {
     wcsstruct		*wcs;
-    OFF_T		offset, pixcount;
+    OFF_T2		offset, pixcount;
     FLAGTYPE		*lineibuf, *linei;
     unsigned int	*multinbuf2,
 			d, y, nflag, naxis;
@@ -1860,11 +2027,14 @@ int	coadd_iload(fieldstruct *field, fieldstruct *wfield,
           pixcount *= wcs->naxisn[d-1];
           ival = rawpos2[d] - wcs->outmin[d];
           if (ival > 0)
-            offset += (OFF_T)ival*pixcount;
+            offset += (OFF_T2)ival*pixcount;
           }
         QFSEEK(field->cat->file,
 		field->tab->bodypos+offset*field->tab->bytepix,
 		SEEK_SET, field->filename);
+#ifdef HAVE_CFITSIO
+        field->tab->currentElement = (offset == 0) ? 1 : offset;
+#endif // HAVE_CFITSIO
         }
 #ifdef USE_THREADS
       linei = lineibuf + (threadstep&1)*field->width;
@@ -1929,6 +2099,9 @@ int	coadd_iload(fieldstruct *field, fieldstruct *wfield,
           QFSEEK(wfield->cat->file,
 		wfield->tab->bodypos+offset*wfield->tab->bytepix,
 		SEEK_SET, wfield->filename);
+#ifdef HAVE_CFITSIO
+          wfield->tab->currentElement = (offset == 0) ? 1 : offset;
+#endif // HAVE_CFITSIO
           }
         read_ibody(wfield->tab, linei, field->width);
         }
@@ -2007,7 +2180,7 @@ int	coadd_load(fieldstruct *field, fieldstruct *wfield,
 			int nbuflines, int outwidth, int multinmax, int oid)
   {
     wcsstruct		*wcs;
-    OFF_T		offset, pixcount;
+    OFF_T2		offset, pixcount;
     PIXTYPE		*linebuf, *line,*linet,
 			thresh;
     unsigned int	*multinbuf2,
@@ -2072,11 +2245,14 @@ int	coadd_load(fieldstruct *field, fieldstruct *wfield,
           pixcount *= wcs->naxisn[d-1];
           ival = rawpos2[d] - wcs->outmin[d];
           if (ival > 0)
-            offset += (OFF_T)ival*pixcount;
+            offset += (OFF_T2)ival*pixcount;
           }
         QFSEEK(field->cat->file,
 		field->tab->bodypos+offset*field->tab->bytepix,
 		SEEK_SET, field->filename);
+#ifdef HAVE_CFITSIO
+        field->tab->currentElement = (offset == 0) ? 1 : offset;
+#endif // HAVE_CFITSIO
         }
 #ifdef USE_THREADS
       line = linebuf+(threadstep&1)*field->width;
@@ -2145,6 +2321,9 @@ int	coadd_load(fieldstruct *field, fieldstruct *wfield,
           QFSEEK(wfield->cat->file,
 		wfield->tab->bodypos+offset*wfield->tab->bytepix,
 		SEEK_SET, wfield->filename);
+#ifdef HAVE_CFITSIO
+      	wfield->tab->currentElement = (offset == 0) ? 1 : offset;
+#endif // HAVE_CFITSIO
           }
         read_body(wfield->tab, line, field->width);
         if ((thresh=wfield->weight_thresh)>0.0)
